#include "rpi_lib/rpi.h"
#include <stdio.h>
#include <stdint.h>


#include "Ab_192.h"

double c[SIZE];
double x[SIZE];

extern void enable_fpu(void);

void spi_print(char *s);
void Serial_send(uint8_t *buf, int len);
void Serial_receive(uint8_t *buf, int len);

char str[100];

void myLUsolver_node1(void){
	// 外積方式ガウス法

	
	int i,j,k;
	int n = SIZE;

	// LU分解
	for (k=0; k<n; k++) {
		// kで回す行方向ループ

		// 行列Lの処理
		if(k < n/2){
			// 枢軸ベクトル受信
			for (i=k+1; i<n; i++) {
				// 開始メッセージ待ち
				while(uart0_getc() != 's');

				Serial_receive((uint8_t *)&A[i][k],sizeof(double)*1);

				// 終了メッセージ送信
				uart0_putc('e');
			}
		}
		else{
			double div_tmp = 1.0 / A[k][k];
			for (i=k+1; i<n; i++) {
				// iで回す行方向ループ

				// l_ik = a_ik / a_kk
				A[i][k] = A[i][k] * div_tmp;   
			}
		}

		// 行列Uの処理
		for (j=k+1; j<n; j++) {
			// jで回す行方向のループ

			// Tips: ここで値を変数に入れたほうが
			// ループ内で参照するよりメモリアクセス回数が減ってお得
			double a_jk = A[j][k];


			if(j < n/2){
				i = n/2;
			}
			else{
				i = k+1;
			}
			for (; i<n; i++) {
				// iで回す列方向のループ
				A[j][i] = A[j][i] - A[k][i] * a_jk; 
			}
		}
	}
	// LU分解終了

	// 終了メッセージ送信
	uart0_putc('e');

	// k/2列目以降のデータを送信
	for(i=0;i<n;i++){
		for(j=n/2;j<n;j++){
			// 開始メッセージ送信
			uart0_putc('s');

			Serial_send((uint8_t *)&A[i][j],sizeof(double)*1);

			// 終了メッセージ待ち
			while(uart0_getc() != 'e');
		}
	}
}

void spi_print(char *s){
	while(*s != '\0'){
		SPI0_transfer(*s);
		s++;
	}
}

void Serial_send(uint8_t *buf, int len){
	int i;
	for(i=0;i<len;i++){
		// 送信FIFOが空くのを待つ
		while(*UART0_FR & (0x01 << 5));
		// データ送信
		int c = buf[i];
		*UART0_DR = c;
	}
}

void Serial_receive(uint8_t *buf, int len){
	int i;
	for(i=0;i<len;i++){
		// データが来るまで待つ
		while(*UART0_FR & (0x01 << 4));
		// データ受信
		int c = *UART0_DR;
		buf[i] = c & 0xff;
	}
}


int main(void){
	rpi_init();
	enable_fpu();

	// UARTボーレート設定
	// baudrate = F_UART / (16*BAUDDIV)
	// baudrate_max = F_UART / (16*0.0625) = F_URAT
	// BAUDDIV = IBRD + ((FBRD - 0.5) / 64)
	// FBRD = int(0.0625 * 64 + 0.5) = 4
	*UART0_IBRD = 0;
	*UART0_FBRD = 0;

	myLUsolver_node1();

	return 0;
}
